#include "Dataset.h"

#include <iostream>
using namespace std;

void Dataset::print() const {
  cout << "========== print dataset ============" <<endl;
  printf("Grid type is %i\n"
	 "Dims: %i %i %i\n",
	 m_gridType,
	 m_dim[0], m_dim[1], m_dim[2]);
  cout << "Scaling is: " << m_scaling << endl;
  cout << "========== end print ================" <<endl;
}

void Dataset::read(const string& fullname) { 
  readDoubles(fullname);
}

void Dataset::write(const string& outDir, const string& filename, const string& ext) const {
  writeBytes(outDir, filename, ext);
  writeDoubles(outDir, filename);
}

void Dataset::writeBytes(const string& outDir, const string& filename, const string& ext) const {
  //WRITE vuVolume dataset
  string bytefile = outDir+filename+string("_byte")+ext;
  FILE* f = fopen(bytefile.c_str(), "wb");
  assert(f);

  cout << "Writing byte dataset to file: " << bytefile << endl;

  //write header
  string gridTypes[] = {"STRUCTURED_POINTS", "BCC_POINTS" }; //CC & BCC
  int dataSize = getNumSamples();
  fprintf(f, "# vu DataFile Version 1.0\n"
	  "Volume Data\n"
	  "BINARY\n"
	  "DATASET %s\n"
	  "UNIMODAL\n"
	  "DIMENSIONS %i %i %i 1\n"
	  "ORIGIN 0 0 0 0\n"
	  "OFFSET 0.000000 0.000000 0.000000 0.000000\n"
	  "SCALING %lf %lf %lf 1\n"
	  "SPACING 0 0 0 0\n"
	  "POINT_DATA %i\n"
	  "SCALARS data byte\n"
	  "LOOKUP_TABLE default\n",
	  gridTypes[m_gridType].c_str(),
	  m_dim[0], m_dim[1], m_dim[2],
	  m_scaling[0], m_scaling[1], m_scaling[2],
	  dataSize
	  );

  //write data: faster to write a chunk of bytes than writing them individually
  static byte data_byte[MAX_NUM];
  int i;
  for ( i = 0; i < dataSize; i++ ) {
    double v = m_data[i];
    assert( v >= 0. && v <= MAX_INTENSITY );
    data_byte[i] = (byte) myRound(v);
    assert( data_byte[i] >= 0. && data_byte[i] <= MAX_INTENSITY );
  }

  int size = fwrite(data_byte,sizeof(byte),dataSize,f);
  printf("Num bytes written is %i \n", size);

  fclose(f);
}

void Dataset::writeDoubles(const string& outDir, const string& filename) const {
  //WRITE my own double-precision dataset
  string doublefile = outDir + filename + string("_double.dat");
  FILE* f = fopen(doublefile.c_str(), "wb");
  assert(f);

  cout << "Writing double dataset to file: " << doublefile << endl;

  //write header
  string gridTypes[] = {"STRUCTURED_POINTS", "BCC_POINTS" }; //CC & BCC
  int dataSize = getNumSamples();
  fprintf(f, 
	  "DATASET %s\n"
	  "DIMENSIONS %i %i %i 1\n"
	  "SCALING %lf %lf %lf 1\n"
	  "POINT_DATA %i\n"
	  "SCALARS data double\n",
	  gridTypes[ m_gridType ].c_str(),
	  m_dim[0], m_dim[1], m_dim[2],
	  m_scaling[0], m_scaling[1], m_scaling[2],
	  dataSize
	  );

  int size = fwrite(m_data,sizeof(double),dataSize,f);
  printf("Num doubles written is %i \n", size);

  fclose(f);
}

void Dataset::readDoubles(const string& fullname) {
  cout << "Reading dataset from file: " << fullname << endl;

  FILE* f = fopen(fullname.c_str(), "rb");
  assert(f);

  //write header
  string gridTypes[] = {"STRUCTURED_POINTS", "BCC_POINTS" }; //CC & BCC
  char gridType[100];
  int dataSize; 
  fscanf(f, 
	"DATASET %s\n"
	"DIMENSIONS %i %i %i 1\n"
	"SCALING %lf %lf %lf 1\n"
	"POINT_DATA %i\n"
	"SCALARS data double\n",
	 gridType,
	 &m_dim[0], &m_dim[1], &m_dim[2],
	 &m_scaling[0], &m_scaling[1], &m_scaling[2],
	 &dataSize
	 );

  if ( gridType == gridTypes[0] ) m_gridType = CC;
  else m_gridType = BCC;

  int size = fread(m_data,sizeof(double),dataSize,f);
  printf("Num scalars read from file is: %i \n", size);

  fclose(f); 
}

void Dataset::copyAttribs(const Dataset& d) {
  m_gridType = d.m_gridType;
  int i;
  for ( i = 0; i < 3; i++ ) {
    m_dim[i] = d.m_dim[i];
  }
  m_scaling = d.m_scaling;
}
