#include <math.h>
#include <stdio.h>
#include <cstdlib> //for rand()
#include <ctime> //for rand seed
#include <assert.h>
#include <iostream>

using namespace std;

#include "vuML.h"

MarschnerLobb::MarschnerLobb()
{
  xsize = ysize = zsize = 1.;
  fm = 6;
  alpha = 0.25;

}

MarschnerLobb::~MarschnerLobb()
{}

void MarschnerLobb::initialize(double x, double y, double z)
{
  xsize = x; ysize = y; zsize = z;
  srand((unsigned)time(NULL));  //init noise
}

double MarschnerLobb::getValue(const vuVector& p) const
{
  double v[3];

  v[0] = 2.0f * p[0]/(double)xsize - 1.0;
  v[1] = 2.0f * p[1]/(double)ysize - 1.0;
  v[2] = 2.0f * p[2]/(double)zsize - 1.0;

  double rhor = cos(2.0f*M_PI*fm*cos(M_PI*sqrt(v[0]*v[0]+v[1]*v[1])/2.0f));
  return MAX_INTENSITY*(1-sin(M_PI*v[2]/2.0f) + alpha *(1.0f+rhor)) / (2.0f*(1.0f+alpha));
}

double MarschnerLobb::getValueGaussianNoise(const vuVector& p, const double& stdDev) const {
  double v = MarschnerLobb::getValue(p);
  
  //Reference: http://www.dspguru.com/howto/tech/wgn.htm
  //The Central Limit Theorm states that the sum of N randoms will
  //approach normal distribution as N approaches infinity. Use N = 30
  //good results.

  //first generate a white noise of mean 0 and variance 1
  const int N = 30;
  double x = 0.0;
  int i;
  for ( i = 0; i < N; i++ ) {
    double u = rand() / double(RAND_MAX); //in range [0,1]
    x += u;
  }
	
  /* for uniform randoms in [0,1], mu = 0.5 and var = 1/12 */
  /* adjust X so mu = 0 and var = 1 */
	
  x -= double(N)/2.0;                /* set mean to 0 */
  x *= sqrt(12.0 / double(N));       /* adjust variance to 1 */
    
  //now x has mean 0 and variance 1, now modify it to have the
  //specified variance
  x *= stdDev; 

  //disallow extreme outliers in gaussian distribution
  if ( fabs(x) > double(MAX_INTENSITY) / 2.0 ) {
    return getValueGaussianNoise(p, stdDev);
  }

  //check for overflow and underflow, reverse affect of x so as to not
  //affect sample  variance.
  if ( v + x > MAX_INTENSITY ||
       v + x < 0 ) {
    return v - x;
  }
  
  return v + x;
}

double MarschnerLobb::getValueSaltPepperNoise(const vuVector& p, const double& prob) const {
  double v = MarschnerLobb::getValue(p);
  
  double randnum = rand() / double(RAND_MAX); //in range [0,1]

  if ( randnum < prob/2.0 ) return 0.0;
  else if ( randnum >= prob/2.0 
	    && randnum <= prob ) return MAX_INTENSITY;
  
  return v;
}

