// -*- C++ -*-
#ifndef __MARSCHNERLOBB_H__
#define __MARSCHNERLOBB_H__

// written by Reza Entezari
// adapted to vuVolume by Steven Bergner
// adapted to 775 final proj by Tai Meng

#include <vuVector.h>

#define MAX_INTENSITY 254 //vuVolume doesn't handle 255 properly

//!Taken partially from vuVolume2. The Marschner Lobb 3D function.
class MarschnerLobb {
public:
  MarschnerLobb();
  ~MarschnerLobb();
  
  //!Scale the function for sampling purposes.
  void initialize(double x, double y, double z);  

  //!Get the function value at p.
  double getValue(const vuVector& p) const;

  //!Add white Guassian noise to function value at p.
  /*!
    The noise has mean 0 and std deviation stdDev.
    Makes use the Central Limit Theorem; the sum of N
    random numbers has an approximately normal distribution w
    hen N is large. N is chosen to be 50.
   */
  double getValueGaussianNoise(const vuVector& p, const double& stdDev) const;

  //!Apply salt & pepper noise model to function value at p.
  /*
    Set value to 0 (pepper) with probability p/2.
    Set value to 1 (salt) with probability p/2.
  */
  double getValueSaltPepperNoise(const vuVector&p, const double& percentage) const;

private:
  double xsize, ysize, zsize;
  double fm, alpha;
};

#endif
