% Tai Meng
% CMPT775 2006-Spring
% Final Project

% This program produces plots for 3 ways of generating white noise:
% - Central Limit Theorem approach
% - rejection sampling approach
% - Matlab built in white noise generator (for comparison)

% Central limit theorem approach:
% - The Central Limit Theorem states that the sum of N random numbers
% will approach normal distribution as N approaches infinity; N >= 30
% works well. 
% - I set N = 50.

% Rejection sampling approach:
% - dart throwing till a sample falls under the Gaussian envelope
% - On average, need to cast 10 pairs of random numbers for each sample
% - For each pair, need to evaluate the Gaussian 1D function (slow)
% - So, noticeably slower than Central limit theorem approach 

% The goal of this program is to choose a white noise generation 
% approach that can be implemented in C, and which runs reasonably quickly. 
% The Central Limit Theorem approach satisfies that criterion.

function main()
num_samples = 20000;
N = 50; 
x = zeros(num_samples,1);
for j = 1:num_samples
    u = sum(rand(N,1));
    x(j) = x(j) + u;
end

x = x - N/2.0;
x = x*sqrt(12.0 / N);

variance=1;
x = x*sqrt(variance);

bins_x = min(x):.2:max(x);
subplot(1,3,1)
hist(x, bins_x)
axis([-5 5 0 num_samples/10])
title('Using Central Limit Theorem.')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

y = zeros(num_samples,1);
sigma=1;
global counter;
counter = 0;
for i = 1:size(y)
    y(i) = getGaussVal(sigma);
end
counter
bins_y = min(y):.2:max(y);
subplot(1,3,2)
hist(y, bins_y)
axis([-5 5 0 num_samples/10])
title('Using Rejection Sampling.')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

y = randn(num_samples,1);
bins_y = min(y):.2:max(y);
subplot(1,3,3)
hist(y, bins_y)
axis([-5 5 0 num_samples/10])
title('Matlab white Gaussian noise.')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function val = getGaussVal(sigma)
    global counter;
    counter = counter+1;
    p = rand(1);  
    x = (rand(1) - 0.5)*10; %between +-10*sigma
    
    if ( p <= gauss1D(x,sigma) )
        val = x;
        return;
    else
        val = getGaussVal(sigma);
    end
end

% gx is the 1D gaussian equation.
function gx = gauss1D(x, sigma)
    gx = 1 / (sqrt(2 * pi) * sigma) * exp(-x^2 / (2 * sigma^2));
end