% Tai Meng
% CMPT775 2006-Spring
% Final Project

% This program traverses a list of hard-coded directories (dirs) and
% makes use of the plot data files contained in those directories to
% produce plots.

% Please see plotData.cpp for a specification of the format of the data
% files.

% For each entry in the ASCII data file, extract two components to produce
% a 2D plot, as follows:
% - entry: filterRadius neighborhoodSize mean stdDev
%   - i.e. 1.000    7   0.01    2.0
% - extracted tuple: (filterRadius stdDev) 
%   - i.e. (1.000    2.0)
% One CC/BCC pair of these plots are overlayed for each noise type,
% and for each noise level.

% For each entry in the ASCII data file, extract two other components to 
% produce another 2D plot, as follows:
% - entry: filterRadius neighborhoodSize mean stdDev
%   - i.e. 1.000    7   0.01    2.0
% - extracted tuple: (filterRadius neighborhoodSize) 
%   - i.e. (1.000    7)
% One CC/BCC pair of these plots are overlayed for each noise level.

% Finally, the the plotted images (in .png format) are saved in the
% directory ../images/stage_2/

function plotData()
    salt_levels = [' 3%'; '10%'; '20%'];
    gauss_levels = [' 2'; ' 7'; '14'];    
    dirs=['01_data_low '; '02_data_med '; '03_data_high'];
    for i=1:3
        dir = strtrim(dirs(i,:));
        dir = ['../common/', dir, '/'];
        
        plotFigure([dir 'plot_cc_salt_no_boundary.dat'], ...
            [dir 'plot_bcc_salt_no_boundary.dat'], ...
            [dir 'plot_cc_gauss_no_boundary.dat'], ...
            [dir 'plot_bcc_gauss_no_boundary.dat'], ...
            [strtrim(salt_levels(i,:)), ' Salt & Pepper Noise, Without Boundary'], ...
            ['White Noise Sigma = ', strtrim(gauss_levels(i,:)), ', Without Boundary'], ...
            ['../images/stage_2/', sprintf('0%i_',i), 'without_boundary.png']);
        
    end
end

function plotFigure(salt_cc, salt_bcc, gauss_cc, gauss_bcc, heading1, heading2, file_out)
    scrsz = get(0,'ScreenSize');
    numRows = 2;
    numCols = 2;
    
    fig=figure('Position',scrsz);
    plotStdDev(salt_cc, numRows, numCols, 1, '', '-*g', 1, 0);
    plotNeighborhood(salt_cc, numRows, numCols, 2, '', '-*g',1);
    plotStdDev(salt_bcc, numRows, numCols, 1, heading1, '-*r',1, 0);
    plotNeighborhood(salt_bcc, numRows, numCols, 2, '','-*r',1);
    
    plotStdDev(gauss_cc, numRows, numCols, 3, '', '-*g', 0, 1);
    plotStdDev(gauss_bcc, numRows, numCols, 3, heading2, '-*r',0, 1);
    
    saveas(fig, file_out);
end

function plotStdDev(filename, numRows, numCols, plot_num, plotTitle,line_format, isLegend, isGauss)
    plotFile=fopen(filename);
    dims=fscanf(plotFile, '%i', [1,2]);
    data=fscanf(plotFile, '%f', dims);    
    fclose(plotFile);
    
    radii=data(1,:);
    stdDev=data(4,:);
    
    axes=subplot(numRows,numCols,plot_num);
    title(plotTitle);
    xlabel('Radius'); ylabel('Standard Deviation');
    if ( isGauss )
        axis([0 3.5 0 15]);
    else %for salt
        axis([0 3.5 0 75]);
    end
    plot(radii,stdDev,line_format);
    if ( isLegend )
        legend(axes,{'CC','BCC'},'Position',[0.249 0.8489 0.08301 0.05469]);
    end
    hold on; %overlay next plot
end

function plotNeighborhood(filename, numRows, numCols, plot_num, plotTitle, line_format, isLegend)
    plotFile=fopen(filename);
    dims=fscanf(plotFile, '%i', [1,2]);
    data=fscanf(plotFile, '%f', dims);    
    fclose(plotFile);
    
    radii=data(1,:);
    neighborhood=data(2,:);
    
    axes=subplot(numRows, numCols,plot_num);
    title(plotTitle);
    xlabel('Radius'); ylabel('Neighborhood Size');
    plot(radii,neighborhood,line_format);
    if ( isLegend )
        legend(axes,{'CC','BCC'},'Position',[0.70 0.8489 0.08301 0.05469]);
    end
    hold on; %overlay next plot
end

